@@ -42,43 +42,6 @@ class ErrorAware(type(QtCore.QObject)):
         
         return super().__new__(meta, classname, bases, new_class_dict)
 
-def promote_to_powder(**kwargs):
-    """ Create a PowderDiffractionDataset from a DiffractionDataset """
-    filename = kwargs.pop('filename')
-    with PowderDiffractionDataset.from_dataset(DiffractionDataset(filename), **kwargs):
-        pass
-    return filename
-
-def recompute_angular_average(**kwargs):
-    """ Re-compute the angular average of a PowderDiffractionDataset """
-    filename = kwargs.pop('filename')
-    with PowderDiffractionDataset(filename, mode = 'r+') as dataset:
-        dataset.compute_angular_averages(**kwargs)
-    return filename
-
-def symmetrize(dataset, destination, **kwargs):
-    """ 
-    Copies a dataset and symmetrize it. Keyword arguments
-    are passed to `DiffractionDataset.symmetrize`.
-    
-    Parameters
-    ----------
-    dataset : path-like
-    destination : path-like
-    """
-    copy2(dataset, destination)
-
-    with DiffractionDataset(destination, 'r+') as dset:
-        dset.symmetrize(**kwargs)
-        fname = dset.filename
-    return fname
-
-def process(**kwargs):
-    """ Process a RawDataset into a DiffractionDataset """
-    with DiffractionDataset.from_raw(**kwargs) as dset:
-        fname = dset.filename
-    return fname
-
 def indicate_in_progress(method):
     """ Decorator for IrisController methods that should
     emit the ``operation_in_progress`` signal and automatically
@@ -115,6 +78,10 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
     time_series_signal              = QtCore.pyqtSignal(object, object)
     powder_time_series_signal       = QtCore.pyqtSignal(object, object)
 
+    relative_powder_enable_signal   = QtCore.pyqtSignal(bool)
+    relative_averaged_enable_signal = QtCore.pyqtSignal(bool)
+    powder_bgr_enable_signal        = QtCore.pyqtSignal(bool)
+
     processing_progress_signal      = QtCore.pyqtSignal(int)
     powder_promotion_progress       = QtCore.pyqtSignal(int)
     angular_average_progress        = QtCore.pyqtSignal(int)
@@ -141,6 +108,7 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         # out parameter
         # TODO: is it worth it?
         self._averaged_data_container = None
+        self._average_time_series_container = None
 
     @QtCore.pyqtSlot(int, int)
     def display_raw_data(self, timedelay_index, scan):
@@ -175,17 +143,14 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         """
         # Preallocation of full images is important because the whole block cannot be
         # loaded into memory, contrary to powder data
-        # Source of 'cache miss' could be that _average_data_container is None,
-        # new dataset loaded has different shape than before, etc.
+        # Note that the containers have been initialized when the dataset was loaded
+        # Therefore, no error checking required.
         self._timedelay_index = timedelay_index
         timedelay = self.dataset.time_points[timedelay_index]
-        try:
-            self._averaged_data_container[:] = self.dataset.diff_data(timedelay, relative = self._relative_averaged)
-        except:
-            self._averaged_data_container = self.dataset.diff_data(timedelay, relative = self._relative_averaged)
+        self._averaged_data_container[:] = self.dataset.diff_data(timedelay, relative = self._relative_averaged, out = self._averaged_data_container)
+        
         self.averaged_data_signal.emit(self._averaged_data_container)
         self.status_message_signal.emit('Displaying data at {:.3f}ps.'.format(timedelay))
-        return self._averaged_data_container
     
     @QtCore.pyqtSlot()
     def display_powder_data(self):
@@ -205,6 +170,7 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         enable : bool
         """
         self._bgr_powder = enable
+        self.powder_bgr_enable_signal.emit(enable)
         self.display_powder_data()
     
     @QtCore.pyqtSlot(bool)
@@ -217,6 +183,7 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         enable : bool
         """
         self._relative_powder = enable
+        self.relative_powder_enable_signal.emit(enable)
         self.display_powder_data()
     
     @QtCore.pyqtSlot(bool)
@@ -229,89 +196,8 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         enable : bool
         """
         self._relative_averaged = enable
+        self.relative_averaged_enable_signal.emit(enable)
         self.display_averaged_data(self._timedelay_index)
-    
-    @QtCore.pyqtSlot(str, dict)
-    def symmetrize(self, destination, params):
-        """ 
-        Launches a background thread that copies the currently-loaded dataset, 
-        symmetrize the copy, and load the copy. 
-        
-        Parameters
-        ----------
-        destination : str or path-like
-            Destination of the symmetrized dataset.
-        params : dict
-            Symmetrization parameters are passed to ``DiffractionDataset.symmetrize``.
-        """
-        kwargs = {'dataset'    : self.dataset.filename,
-                  'destination': destination,
-                  'callback': self.processing_progress_signal.emit}
-        kwargs.update(params)
-        self.dataset.close()
-
-        self.worker = WorkThread(function = symmetrize, kwargs = kwargs)
-        self.worker.results_signal.connect(self.load_dataset)
-        self.worker.in_progress_signal.connect(self.operation_in_progress)
-        self.worker.done_signal.connect(lambda boolean: self.processing_progress_signal.emit(100))
-        self.processing_progress_signal.emit(0)
-        self.worker.start()
-
-    @QtCore.pyqtSlot(dict)
-    def process_raw_dataset(self, info_dict):
-        """
-        Launch a background thread that reduces raw data.
-
-        Parameters
-        ----------
-        info_dict : dict
-            Data reduction parameters are passed to ``DiffractionDataset.from_raw``
-        """
-        info_dict.update({'callback': self.processing_progress_signal.emit, 
-                          'raw': self.raw_dataset})
-
-        self.worker = WorkThread(function = process, kwargs = info_dict)
-        self.worker.results_signal.connect(self.load_dataset)
-        self.worker.in_progress_signal.connect(self.operation_in_progress)
-        self.worker.done_signal.connect(lambda boolean: self.processing_progress_signal.emit(100))
-        self.processing_progress_signal.emit(0)
-        self.worker.start()
-    
-    @QtCore.pyqtSlot(dict)
-    def promote_to_powder(self, params):
-        """ 
-        Promote a DiffractionDataset to a PowderDiffractionDataset 
-        
-        Parameters
-        ----------
-        params : dict
-            Parameters are passed to ``PowderDiffractionDataset.from_dataset``.
-        """
-        params.update({'filename':self.dataset.filename, 'callback':self.powder_promotion_progress.emit})
-        self.worker = WorkThread(function = promote_to_powder, kwargs = params)
-        self.dataset.close()
-        self.dataset = None
-        self.worker.results_signal.connect(self.load_dataset)
-        self.worker.in_progress_signal.connect(self.operation_in_progress)
-        self.worker.start()
-    
-    @QtCore.pyqtSlot(dict)
-    def recompute_angular_average(self, params):
-        """ 
-        Compute the angular average of a PowderDiffractionDataset again.
-        
-        Parameters
-        ----------
-        params : dict
-            Parameters are passed to ``PowderDiffractionDataset.compute_angular_averages``. 
-        """
-        params.update({'filename':self.dataset.filename, 'callback':self.powder_promotion_progress.emit})
-        self.worker = WorkThread(function = recompute_angular_average, kwargs = params)
-        self.dataset.close()
-        self.dataset = None
-        self.worker.results_signal.connect(self.load_dataset)
-        self.worker.in_progress_signal.connect(self.operation_in_progress)
-        self.worker.start()
 
     @QtCore.pyqtSlot(float, float)
     def powder_time_series(self, qmin, qmax):
@@ -326,7 +212,7 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         time_series = self.dataset.powder_time_series(rmin = qmin, rmax = qmax, bgr = self._bgr_powder, units = 'momentum')
         self.powder_time_series_signal.emit(self.dataset.time_points, time_series)
     
-    @QtCore.pyqtSlot(object)
+    @QtCore.pyqtSlot(tuple)
     def time_series(self, rect):
         """" 
         Single-crystal time-series as the integrated diffracted intensity inside a rectangular ROI
@@ -339,14 +225,10 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         # Remember for updates
         self._rect = rect
 
-        #If coordinate is negative, return 0
-        x1 = round(max(0, rect.topLeft().x() ))
-        x2 = round(max(0, rect.x() + rect.width() ))
-        y1 = round(max(0, rect.topLeft().y() ))
-        y2 = round(max(0, rect.y() + rect.height() ))
-
-        integrated = self.dataset.time_series( (x1, x2, y1, y2) )
-        self.time_series_signal.emit(self.dataset.time_points, integrated)
+        # Note that the containers have been initialized when the dataset was loaded
+        # Therefore, no error checking required.
+        integrated = self.dataset.time_series( rect, out = self._average_time_series_container )
+        self.time_series_signal.emit(self.dataset.time_points, self._average_time_series_container)
     
     @QtCore.pyqtSlot(dict)
     def powder_calq(self, params):
@@ -361,19 +243,6 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         self.dataset.powder_calq(**params)
         self.display_powder_data()
         self.status_message_signal.emit('Scattering vector range calibrated.')
-
-    @QtCore.pyqtSlot(dict)
-    def compute_baseline(self, params):
-        """ Compute the powder baseline. The dictionary `params` is passed to 
-        PowderDiffractionDataset.compute_baseline(), except its key 'callback'. The callable 'callback' 
-        is called (no argument) when computation is done. """
-        callback = params.pop('callback')
-        self.worker = WorkThread(function = self.dataset.compute_baseline, kwargs = params)
-        self.worker.done_signal.connect(lambda b: callback())
-        self.worker.done_signal.connect(lambda b: self.dataset_metadata.emit(self.dataset.metadata))
-        self.worker.done_signal.connect(lambda b: self.display_powder_data())
-        self.worker.in_progress_signal.connect(self.operation_in_progress)
-        self.worker.start()
     
     @QtCore.pyqtSlot(dict)
     def update_metadata(self, metadata):
@@ -388,10 +257,20 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         if self.dataset is None:
             return
 
-        for key, val in metadata.items():
-            setattr(self.dataset, key, val)
+        # dataset is read-only at this point
+        # We close and reopen with write intentions
+        fname = self.dataset.filename
+        cls = type(self.dataset)
+        self.dataset.close()
+
+        with cls(fname, mode = 'r+') as dset:
+            for key, val in metadata.items():
+                if key in dset.valid_metadata:
+                    setattr(dset, key, val)
+        
+        self.dataset = cls(fname, mode = 'r')
+        self.dataset_metadata.emit(self.dataset.metadata)
         
-        self.load_dataset(self.dataset.filename)
         self.status_message_signal.emit('Metadata updated.')
 
     @QtCore.pyqtSlot(str)
@@ -403,7 +282,9 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         ----------
         notes : str
         """
-        self.dataset.notes = notes
+        metadata = self.dataset.metadata
+        metadata['notes'] = notes
+        self.update_metadata(metadata)
     
     @QtCore.pyqtSlot(float)
     def set_time_zero_shift(self, shift):
@@ -418,7 +299,17 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         if shift == self.dataset.time_zero_shift:
             return
 
-        self.dataset.shift_time_zero(shift)
+        # dataset is read-only at this point
+        # We close and reopen with write intentions
+        # Note : no need to open as a PowderDiffractionDataset for this operations
+        fname = self.dataset.filename
+        cls = type(self.dataset)
+        self.dataset.close()
+
+        with DiffractionDataset(fname, mode = 'r+') as dset:
+            dset.shift_time_zero(shift)
+
+        self.dataset = cls(fname, mode = 'r')
         self.dataset_metadata.emit(self.dataset.metadata)
 
         # In case of a time-zero shift, diffraction time-series will
@@ -468,8 +359,7 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         self.raw_data_signal.emit(None)
 
         self.status_message_signal.emit('Raw dataset closed.')
-        
-    @QtCore.pyqtSlot(object) # Due to worker.results_signal emitting an object
+
     @QtCore.pyqtSlot(str)
     @indicate_in_progress
     def load_dataset(self, path):
@@ -490,16 +380,30 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
         with DiffractionDataset(path, mode = 'r') as d:
             if PowderDiffractionDataset._powder_group_name in d:
                 cls = PowderDiffractionDataset
-        
-        self.dataset = cls(path, mode = 'r+')
-        self.dataset_metadata.emit(self.dataset.metadata)
-        self.processed_dataset_loaded_signal.emit(True)
-        self.display_averaged_data(timedelay_index = 0)
+                is_powder = True
+            else:
+                is_powder = False
 
-        if isinstance(self.dataset, PowderDiffractionDataset):
-            self.display_powder_data()
-            self.powder_dataset_loaded_signal.emit(True)
+        self.dataset = cls(path, mode = 'r')
+        self.dataset_metadata.emit(self.dataset.metadata)
+
+        # Initialize containers
+        # This *must* be done before data is displayed
+        self._averaged_data_container = np.empty(shape = self.dataset.resolution, 
+                                                 dtype = self.dataset.diffraction_group['intensity'].dtype)
+        self._average_time_series_container = np.empty(shape = self.dataset.time_points.shape, dtype = np.float)
         
+        self.processed_dataset_loaded_signal.emit(True)
+        self.powder_dataset_loaded_signal.emit(is_powder)
+
+        self.display_averaged_data(timedelay_index = 0)
+        if is_powder:
+            self.display_powder_data()
+        else:
+            # Re-assert that a processed dataset was loaded
+            # hence, switch views to the relevant widgets
+            self.processed_dataset_loaded_signal.emit(True)
+
         self.status_message_signal.emit(path + ' loaded.')
     
     @QtCore.pyqtSlot()
@@ -517,9 +421,111 @@ class IrisController(QtCore.QObject, metaclass = ErrorAware):
 
         self.status_message_signal.emit('Dataset closed.')
 
+    @QtCore.pyqtSlot(str, dict)
+    def symmetrize(self, destination, params):
+        """ 
+        Launches a background thread that copies the currently-loaded dataset, 
+        symmetrize the copy, and load the copy. 
+        
+        Parameters
+        ----------
+        destination : str or path-like
+            Destination of the symmetrized dataset.
+        params : dict
+            Symmetrization parameters are passed to ``DiffractionDataset.symmetrize``.
+        """
+        kwargs = {'dataset'    : self.dataset.filename,
+                  'destination': destination,
+                  'callback': self.processing_progress_signal.emit}
+        kwargs.update(params)
+        self.dataset.close()
+
+        self.worker = WorkThread(function = symmetrize, kwargs = kwargs)
+        self.worker.results_signal.connect(self.load_dataset)
+        self.worker.in_progress_signal.connect(self.operation_in_progress)
+        self.worker.done_signal.connect(lambda : self.processing_progress_signal.emit(100))
+        self.processing_progress_signal.emit(0)
+        self.worker.start()
+
+    @QtCore.pyqtSlot(dict)
+    def process_raw_dataset(self, info_dict):
+        """
+        Launch a background thread that reduces raw data.
+
+        Parameters
+        ----------
+        info_dict : dict
+            Data reduction parameters are passed to ``DiffractionDataset.from_raw``
+        """
+        info_dict.update({'callback': self.processing_progress_signal.emit, 
+                          'raw': self.raw_dataset})
+
+        self.worker = WorkThread(function = process, kwargs = info_dict)
+        self.worker.results_signal.connect(self.load_dataset)
+        self.worker.in_progress_signal.connect(self.operation_in_progress)
+        self.worker.done_signal.connect(lambda : self.processing_progress_signal.emit(100))
+        self.processing_progress_signal.emit(0)
+        self.worker.start()
+    
+    @QtCore.pyqtSlot(dict)
+    def promote_to_powder(self, params):
+        """ 
+        Promote a DiffractionDataset to a PowderDiffractionDataset 
+        
+        Parameters
+        ----------
+        params : dict
+            Parameters are passed to ``PowderDiffractionDataset.from_dataset``.
+        """
+        params.update({'filename':self.dataset.filename, 'callback':self.powder_promotion_progress.emit})
+        self.worker = WorkThread(function = promote_to_powder, kwargs = params)
+        self.dataset.close()
+        self.dataset = None
+        self.worker.results_signal.connect(self.load_dataset)
+        self.worker.in_progress_signal.connect(self.operation_in_progress)
+        self.worker.start()
+    
+    @QtCore.pyqtSlot(dict)
+    def recompute_angular_average(self, params):
+        """ 
+        Compute the angular average of a PowderDiffractionDataset again.
+        
+        Parameters
+        ----------
+        params : dict
+            Parameters are passed to ``PowderDiffractionDataset.compute_angular_averages``. 
+        """
+        params.update({'filename':self.dataset.filename, 
+                       'callback':self.powder_promotion_progress.emit})
+        self.worker = WorkThread(function = recompute_angular_average, kwargs = params)
+        self.dataset.close()
+        self.dataset = None
+        self.worker.results_signal.connect(self.load_dataset)
+        self.worker.in_progress_signal.connect(self.operation_in_progress)
+        self.worker.start()
+
+
+    @QtCore.pyqtSlot(dict)
+    def compute_baseline(self, params):
+        """ Compute the powder baseline. The dictionary `params` is passed to 
+        PowderDiffractionDataset.compute_baseline(), except its key 'callback'. The callable 'callback' 
+        is called (no argument) when computation is done. """
+        params.update({'fname': self.dataset.filename})
+
+        self.worker = WorkThread(function = compute_powder_baseline, kwargs = params)
+        self.dataset.close()
+        self.dataset = None
+
+        self.worker.results_signal.connect(self.load_dataset)
+        self.worker.done_signal.connect(lambda : self.powder_background_subtracted(True))
+        self.worker.in_progress_signal.connect(self.operation_in_progress)
+        self.worker.start()
+
 class WorkThread(QtCore.QThread):
     """
-    Object taking care of threading computations.
+    Object taking care of threading computations. These computations are very specific:
+    a function takes in a filename (pointing to a DiffractionDataset), parameters, 
+    and returns a filename again.
     
     Signals
     -------
@@ -530,28 +536,9 @@ class WorkThread(QtCore.QThread):
     results_signal
         Emitted when the function evaluation is over. Carries the results
         of the computation.
-        
-    Attributes
-    ----------
-    function : callable
-        Function to be called
-    args : tuple
-        Positional arguments of function
-    kwargs : dict
-        Keyword arguments of function
-    results : object
-        Results of the computation
-    
-    Examples
-    --------
-    >>> function = lambda x : x ** 10
-    >>> result_function = lambda x: print(x)
-    >>> worker = WorkThread(function, 2)  # 2 ** 10
-    >>> worker.results_signal.connect(result_function)
-    >>> worker.start()      # Computation starts only when this method is called
     """
     results_signal = QtCore.pyqtSignal(str)
-    done_signal = QtCore.pyqtSignal(bool)
+    done_signal = QtCore.pyqtSignal()
     in_progress_signal = QtCore.pyqtSignal(bool)
 
     def __init__(self, function, args = tuple(), kwargs = dict()):
@@ -567,5 +554,56 @@ class WorkThread(QtCore.QThread):
     def run(self):
         self.in_progress_signal.emit(True)
         result = self.function(*self.args, **self.kwargs)   
-        self.done_signal.emit(True)  
         self.results_signal.emit(result)
+        self.done_signal.emit()  
+
+
+def promote_to_powder(**kwargs):
+    """ Create a PowderDiffractionDataset from a DiffractionDataset """
+    filename = kwargs.pop('filename')
+    with PowderDiffractionDataset.from_dataset(DiffractionDataset(filename), **kwargs):
+        pass
+    return filename
+
+def recompute_angular_average(**kwargs):
+    """ Re-compute the angular average of a PowderDiffractionDataset """
+    filename = kwargs.pop('filename')
+    with PowderDiffractionDataset(filename, mode = 'r+') as dataset:
+        dataset.compute_angular_averages(**kwargs)
+    return filename
+
+def symmetrize(dataset, destination, **kwargs):
+    """ 
+    Copies a dataset and symmetrize it. Keyword arguments
+    are passed to `DiffractionDataset.symmetrize`.
+    
+    Parameters
+    ----------
+    dataset : path-like
+    destination : path-like
+    """
+    copy2(dataset, destination)
+
+    with DiffractionDataset(destination, mode = 'r+') as dset:
+        dset.symmetrize(**kwargs)
+        fname = dset.filename
+    return fname
+
+def compute_powder_baseline(fname, **kwargs):
+    """ 
+    Compute a powder baseline. Keyword arguments are passed to 
+    `PowderDiffractionDataset.compute_baseline` 
+    
+    Parameters
+    ----------
+    dataset : path-like
+    """
+    with PowderDiffractionDataset(fname, mode = 'r+') as dset:
+        dset.compute_baseline(**kwargs)
+    return fname
+
+def process(**kwargs):
+    """ Process a RawDataset into a DiffractionDataset """
+    with DiffractionDataset.from_raw(**kwargs) as dset:
+        fname = dset.filename
+    return fname
\ No newline at end of file
