@@ -28,8 +28,7 @@ class McGillRawDataset(AbstractRawDataset):
         metadata_dict = self.parse_metadata(join(source, 'metadata.cfg'))
         super().__init__(source, metadata_dict)
 
-    @staticmethod
-    def parse_metadata(fname):
+    def parse_metadata(self, fname):
         """ 
         Translate metadata from experiment into Iris's metadata format. 
         
@@ -46,7 +45,7 @@ class McGillRawDataset(AbstractRawDataset):
         
         # Translation is required between metadata formats
         metadata['energy']          = exp_params['electron energy']
-        metadata['date']            = exp_params['acquisition date']
+        metadata['acquisition_date']= exp_params['acquisition date']
         metadata['fluence']         = exp_params['fluence']
         metadata['temperature']     = exp_params['temperature']
         metadata['exposure']        = exp_params['exposure']
